ggkm <- function(sfit,
                 table = TRUE,
                 returns = FALSE,
                 xlabs = "Time",
                 ylabs = "Survival Probability",
                 xlims = c(0,max(sfit$time)),
                 xlimsRestrict = FALSE,
                 maxX = NULL,
                 addHR=addHR,
                 hr=hr,
                 addCI=addCI,
                 ci=ci,
                 ylims = c(0,1),
                 ystratalabs = NULL,
                 ystrataname = NULL,
                 timeby = 100,
                 main = "Kaplan-Meier Plot",
                 pval = TRUE,
                 subs = NULL,
                 xval,
                 yval,
                 legend_x1=NULL,
                 legend_x2=NULL,
#                  legend_y1 = 0.85,
#                  legend_y2 = 0.80,
                 legend_y1 = NULL,
                 legend_y2 = NULL,
                 table_text_size = 3.5,
                 plot.theme.margin.bottom=0.5,
                 plot.theme.margin.lefta=1.5,
                 plot.theme.margin.leftb=2.5,
                 data.table.margin.top=-1.5,
                 data.table.margin.right=1.0,
                 data.table.margin.bottom=0.1,
                 data.table.margin.lefta=2.5,
                 data.table.margin.leftb=3.5,
                 grid.height.p=2.0,
                 grid.height.blank.pic=0.1,
                 grid.height.data.table=0.25,
                 gridheightunit1="null",
                 gridheightunit2="null",
                 gridheightunit3="null",
                 ...) {

  if(xlimsRestrict){
     xlims = c(0, maxX)
  }
  if(is.null(legend_y1)) legend_y1 = 0.85
  if(is.null(legend_y2)) legend_y2 = 0.80
  if(is.null(legend_x1)) legend_x1 = 0.85
  if(is.null(legend_x2)) legend_x2 = 0.75

  #############
  # libraries #
  #############
  
  #Check if the following packages have been installed. If not, install them
  if (!"ggplot2" %in% installed.packages()) install.packages("ggplot2")
  if (!"survival" %in% installed.packages()) install.packages("survival")
  if (!"gridExtra" %in% installed.packages()) install.packages("gridExtra")
  if (!"reshape" %in% installed.packages()) install.packages("reshape")
  
  suppressPackageStartupMessages(library(ggplot2, warn.conflicts=FALSE))
  suppressPackageStartupMessages(library(survival, warn.conflicts=FALSE))
  suppressPackageStartupMessages(library(gridExtra, warn.conflicts=FALSE))
  suppressPackageStartupMessages(library(reshape, warn.conflicts=FALSE))
  suppressPackageStartupMessages(library(plyr, warn.conflicts=FALSE))
  
  #################################
  # sorting the use of subsetting #
  #################################
  
  times <- seq(0, max(sfit$time), by = timeby)
  
  if(is.null(subs)){
    if(length(levels(summary(sfit)$strata)) == 0) {
      subs1 <- 1
      subs2 <- 1:length(summary(sfit,censored=T)$time)
      subs3 <- 1:length(summary(sfit,times = times,extend = TRUE)$time)
    } else {
      subs1 <- 1:length(levels(summary(sfit)$strata))
      subs2 <- 1:length(summary(sfit,censored=T)$strata)
      subs3 <- 1:length(summary(sfit,times = times,extend = TRUE)$strata)
    }
  } else {
    for(i in 1:length(subs)){
      if(i==1){
        ssvar <- paste("(?=.*\\b=",subs[i],sep="")
      }
      if(i==length(subs)){
        ssvar <- paste(ssvar,"\\b)(?=.*\\b=",subs[i],"\\b)",sep="")
      }
      if(!i %in% c(1, length(subs))){
        ssvar <- paste(ssvar,"\\b)(?=.*\\b=",subs[i],sep="")
      }
      if(i==1 & i==length(subs)){
        ssvar <- paste("(?=.*\\b=",subs[i],"\\b)",sep="")
      }
    }
    subs1 <- which(regexpr(ssvar,levels(summary(sfit)$strata), perl=T)!=-1)
    subs2 <- which(regexpr(ssvar,summary(sfit,censored=T)$strata, perl=T)!=-1)
    subs3 <- which(regexpr(ssvar,summary(sfit,times = times,extend = TRUE)$strata, perl=T)!=-1)
  }
  
  if( !is.null(subs) ) pval <- FALSE
  
  ##################################
  # data manipulation pre-plotting #
  ##################################
  
  if(length(levels(summary(sfit)$strata)) == 0) {
    #[subs1]
    if(is.null(ystratalabs)) ystratalabs <- as.character(sub("group=*","","All"))
  } else {
    #[subs1]
    if(is.null(ystratalabs)) ystratalabs <- as.character(sub("group=*","",names(sfit$strata)))
  }
  
  if(is.null(ystrataname)) ystrataname <- "Strata"
  m <- max(nchar(ystratalabs))
  times <- seq(0, max(sfit$time), by = timeby)
  
  if(length(levels(summary(sfit)$strata)) == 0) {
    Factor <- factor(rep("All",length(subs2)))
  } else {
    Factor <- factor(summary(sfit, censored = T)$strata[subs2])
  }
  
  #Data to be used in the survival plot
  .df <- data.frame(
    time = sfit$time[subs2],
    n.risk = sfit$n.risk[subs2],
    n.event = sfit$n.event[subs2],
    surv = sfit$surv[subs2],
    strata = Factor,
    upper = sfit$upper[subs2],
    lower = sfit$lower[subs2]
  )

  #Final changes to data for survival plot
  levels(.df$strata) <- ystratalabs
  zeros <- data.frame(time = 0, surv = 1,
                      strata = factor(ystratalabs, levels=levels(.df$strata)),
                      upper = 1, lower = 1)
  .df <- rbind.fill(zeros, .df)
  d <- length(levels(.df$strata))

  ###################################
  # specifying plot parameteres etc #
  ###################################
  lty_full <- c(1,2,3,4)
  lty <- lty_full[seq(1,length(levels(.df$strata)))]
  p <- ggplot( .df, aes(time, surv)) +
#     geom_step(aes(linetype = strata, colour = strata), size = 0.7) +
#     geom_step(aes(colour = strata), size = 0.7) +
    geom_step(aes(linetype = strata), size = 0.7) +
    theme_bw() +
    theme(axis.title.x = element_text(vjust = 0.5)) +
    theme(axis.text = element_text(size=12)) +
    theme(axis.title = element_text(size=14)) +
    scale_x_continuous(xlabs, breaks = times, limits = xlims) +
    scale_y_continuous(ylabs, limits = ylims) +
    theme(panel.grid.minor = element_blank()) +
    # MOVE LEGEND HERE BELOW [first is x dim, second is y dim]
    # Commented out is original positioning
    #theme(legend.position = c(ifelse(m < 10, .85, .75),ifelse(d < 4, .85, .8))) +
#     theme(legend.position = c(ifelse(m < 10, .85, .75),ifelse(d < 4, legend_y1, legend_y2))) +
    theme(legend.position = c(ifelse(m < 10, legend_x1, legend_x2),ifelse(d < 4, legend_y1, legend_y2))) +
#     theme(legend.key = element_rect(colour = NA)) +
    theme(legend.title = element_blank()) + # element_text(size=12)
    theme(legend.text = element_text(size=8)) + # size was 12

    labs(linetype = ystrataname) +
#     labs(colour = ystrataname) +

    theme(plot.margin = unit(c(0, 1, plot.theme.margin.bottom,ifelse(m < 10, plot.theme.margin.lefta, plot.theme.margin.leftb)),"lines")) +

#     ## testing
#     theme(plot.margin = unit(c(0, 1, 1.0,ifelse(m < 10, 0.5, 1.5)),"lines")) +

    ggtitle(main)
#     +
#     scale_linetype_manual(values=lty) # Change linetypes


  ## Create a blank plot for place-holding
  blank.pic <- ggplot(.df, aes(time, surv)) +
    geom_blank() + theme_bw() +
    theme(axis.text.x = element_blank(),axis.text.y = element_blank(),
          axis.title.x = element_blank(),axis.title.y = element_blank(),
          axis.ticks = element_blank(),
          panel.grid.major = element_blank(),panel.border = element_blank())

  #####################
  # p-value placement #
  #####################a

  if(length(levels(summary(sfit)$strata)) == 0) pval <- FALSE

  if(pval) {
#     sdiff <- survdiff(eval(sfit$call$formula), data = eval(sfit$call$data))
    ###  Modified 2/16/2016 ####
    altfit <- sfit$call
    altfit[[1]] <- quote(survdiff)
    sdiff <- eval.parent(altfit,1)
    #############################
    pval <- pchisq(sdiff$chisq,length(sdiff$n) - 1,lower.tail = FALSE)
#    pvaltxt <- ifelse(pval < 0.0001,"p < 0.0001",paste("p =", signif(pval, 3)))
    pvaltxt <- ifelse(pval < 0.001,"p < 0.001",
                      ifelse(pval >= 0.001 & pval < 0.01,
                      paste('p = ',format(round(pval,3),nsmall=3)),
                      paste("p =", format(round(pval, 2),nsmall=2))))
    if(addHR){
       hrtxt <- paste0('HR = ',hr)
    } else {
       hrtxt <- NULL
    }
    if(addCI){
       citxt <- paste0('95% CI = ',ci)
    } else {
       citxt <- NULL
    }

     # MOVE P-VALUE LEGEND HERE BELOW [set x and y]
    p <- p + annotate("text",x = xval, y = yval + 0.10, label = hrtxt, size=3.25)
    p <- p + annotate("text",x = xval, y = yval + 0.05, label = citxt, size = 3.25)
    p <- p + annotate("text",x = xval, y = yval,label = pvaltxt,size=3.25)
  }

  ###################################################
  # Create table graphic to include at-risk numbers #
  ###################################################

  if(length(levels(summary(sfit)$strata)) == 0) {
    Factor <- factor(rep("All",length(subs3)))
  } else {
    Factor <- factor(summary(sfit,times = times,extend = TRUE)$strata[subs3])
  }

  if(table) {
    risk.data <- data.frame(
      strata = Factor,
      time = summary(sfit,times = times,extend = TRUE)$time[subs3],
      n.risk = summary(sfit,times = times,extend = TRUE)$n.risk[subs3]
    )
    risk.data$strata <- factor(risk.data$strata, levels=rev(levels(risk.data$strata)))

    data.table <- ggplot(risk.data,aes(x = time, y = strata, label = format(n.risk, nsmall = 0))) +
      geom_text(size = table_text_size) + theme_bw() + # was size=3.5
      scale_y_discrete(breaks = as.character(levels(risk.data$strata)),
                       labels = rev(ystratalabs)) +
      scale_x_continuous("Numbers at risk", limits = xlims) +
      theme(axis.title.x = element_text(size = 10, vjust = 1), # size was 12
            panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
            panel.border = element_blank(),axis.text.x = element_blank(),
            axis.ticks = element_blank(),axis.text.y = element_text(face = "bold",hjust = 1))

    data.table <- data.table +
      theme(legend.position = "none") + xlab(NULL) + ylab(NULL)

    # ADJUST POSITION OF TABLE FOR AT RISK
    ## original...
    data.table <- data.table +
      theme(plot.margin = unit(c(data.table.margin.top, data.table.margin.right, data.table.margin.bottom, ifelse(m < 10, data.table.margin.lefta, data.table.margin.leftb) - 0.15 * m), "lines"))
    ### testing
#     data.table <- data.table +
#       theme(plot.margin = unit(c(0, 1, 0, ifelse(m < 10, 1.5, 2.5) - 0.15 * m), "lines"))

    #######################
    # Plotting the graphs #
    #######################

    grid.arrange(p, blank.pic, data.table, clip = FALSE, nrow = 3,
                 ncol = 1, heights = unit(c(grid.height.p, grid.height.blank.pic, grid.height.data.table),c(gridheightunit1, gridheightunit2, gridheightunit3))) # was 0.25 rather than 0.75; first was 2 rather than 2.25

    if(returns) {
      a <- arrangeGrob(p, blank.pic, data.table, clip = FALSE, nrow = 3,
                       ncol = 1, heights = unit(c(2, .1, .25), c("null", "null", "null")))
      return(a)
    }#if
  } else {
    if(returns) return(p)
  }#else
}#function ggkm
