ce_tb_tmp <- ce_tb

# Adding in sex from table basic
ce_tb_tmp$male <- basic[match(ce_tb_tmp$unique_id, basic$unique_id, nomatch=NA),'male']
ce_tb_tmp$male.factor <- factor(ce_tb_tmp$male, levels=c(0,1), labels=c('Female','Male'))

# Creating indicator of those included because they were on the correct TB treatments vs not:
ce_tb_tmp$included_based_on_therapy <- ifelse(ce_tb_tmp$unique_id %in% unique(std_tb_after_therapy_exclusion$unique_id),1,0)
ce_tb_tmp$included_based_on_therapy.factor <- factor(ce_tb_tmp$included_based_on_therapy,
                                                     levels=c(0,1),
                                                     labels=c('Excluded',
                                                              'Included'))
label(ce_tb_tmp$included_based_on_therapy.factor) <- 'Received standard TB therapy (initiation and continuation)'                                                              

# Removing TB case before baseline_d
ce_tb_tmp$baseline_d <- basic[match(ce_tb_tmp$unique_id, basic$unique_id, nomatch=NA),'baseline_d']
ce_tb_tmp$baseline_d <- as.Date(ce_tb_tmp$baseline_d, format='%Y-%m-%d')

ce_tb_tmp2 <- subset(ce_tb_tmp, baseline_d <= tbdiagnosis_d)

# Need to remove duplicate
ce_tb_tmp2 <- ce_tb_tmp2[order(ce_tb_tmp2$unique_id, ce_tb_tmp2$tbdiagnosis_d),]
ce_tb_tmp2$num_tb_dx <- NA
split(ce_tb_tmp2$num_tb_dx, ce_tb_tmp2$unique_id) <- lapply(split(ce_tb_tmp2[,c('unique_id','tbdiagnosis_d')], ce_tb_tmp2[,'unique_id']), FUN=function(y){
                                                seq(1,nrow(y)) })
ce_tb_tmp2$total_num_tb_dx <- unlist(tapply(ce_tb_tmp2$unique_id, ce_tb_tmp2$unique_id, FUN=function(y){
                             rep(length(y),times=length(y)) }))

first_tbdiagnosis_d <- tapply(ce_tb_tmp2$tbdiagnosis_d, ce_tb_tmp2$unique_id, FUN=function(y){ min(y) })
first_tbdiagnosis_d.df <- data.frame('unique_id'=names(first_tbdiagnosis_d),
                                     'first_tbdiagnosis_d'=first_tbdiagnosis_d)
ce_tb_tmp2$first_tbdiagnosis_d <- first_tbdiagnosis_d.df[match(ce_tb_tmp2$unique_id, first_tbdiagnosis_d.df$unique_id, nomatch=NA),'first_tbdiagnosis_d']
ce_tb_tmp2$first_tbdiagnosis_d <- as.Date(ce_tb_tmp2$first_tbdiagnosis_d, origin='1970-01-01')


ce_tb_tmp2$time_from_first_tb_dx <- as.numeric(ce_tb_tmp2$tbdiagnosis_d - ce_tb_tmp2$first_tbdiagnosis_d)
ce_tb_tmp2$max_tbdiagnosis_d <- unlist(tapply(ce_tb_tmp2$tbdiagnosis_d, ce_tb_tmp2$unique_id, FUN=function(y){
                               n <- length(y)
                               rep(y[n],times=n) }))
ce_tb_tmp2$max_tbdiagnosis_d <- as.Date(ce_tb_tmp2$max_tbdiagnosis_d, origin='1970-01-01')

# Subsetting out those subsequent TB diagnoses that are within 180 days of the first TB diagnosis
which.rows <- ce_tb_tmp2$time_from_first_tb_dx <= 180 & ce_tb_tmp2$num_tb_dx > 1
ce_tb_tmp3 <- ce_tb_tmp2[!which.rows,]

# Renumbering now that those recurrent TB diagnoses within 180 days of the first were removed:
ce_tb_tmp3 <- ce_tb_tmp3[order(ce_tb_tmp3$unique_id, ce_tb_tmp3$tbdiagnosis_d),]
ce_tb_tmp3$num_tb_dx <- NA
split(ce_tb_tmp3$num_tb_dx, ce_tb_tmp3$unique_id) <- lapply(split(ce_tb_tmp3[,c('unique_id','tbdiagnosis_d')], ce_tb_tmp3[,'unique_id']), FUN=function(y){
                                                seq(1,nrow(y)) })
ce_tb_tmp3$total_num_tb_dx <- unlist(tapply(ce_tb_tmp3$unique_id, ce_tb_tmp3$unique_id, FUN=function(y){
                             rep(length(y),times=length(y)) }))

# And now reducing the data to one record per subject
ce_tb_tmp3 <- ce_tb_tmp3[order(ce_tb_tmp3$unique_id, ce_tb_tmp3$tbdiagnosis_d),]
ce_tb_tmp4 <- subset(ce_tb_tmp3, !duplicated(unique_id))

# Merging in tbdiagnosis_d to the art table to determine at which point (before, during, after) the TB diagnosis occurred relative to the ART regimen
# First want to subset ART so that it only includes those subjects who had standard TB therapy
sub_artA <- subset(art, unique_id %in% ce_tb_tmp4$unique_id & art_class == 'HAART')
sub_artA$tbdiagnosis_d <- ce_tb_tmp4[match(sub_artA$unique_id, ce_tb_tmp4$unique_id, nomatch=NA),'tbdiagnosis_d']
sub_artA$tbtx_cont_stop_d <- ce_tb_tmp4[match(sub_artA$unique_id, ce_tb_tmp4$unique_id, nomatch=NA),'tbtx_cont_stop_d']
sub_artA$l_alive_d <- follow[match(sub_artA$unique_id, follow$unique_id, nomatch=NA),'l_alive_d']
sub_artA$l_alive_d <- as.Date(sub_artA$l_alive_d, format='%Y-%m-%d')
sub_artA$art_sd <- as.Date(sub_artA$art_sd, format='%Y-%m-%d')
sub_artA$art_ed <- as.Date(sub_artA$art_ed, format='%Y-%m-%d')

# There are some subjects whose tbtx_cont_stop_d are missing but who died within 180 days of TB dx.  Will replace tbtx_cont_stop_d with the date of death
sub_artA$death_d <- follow[match(sub_artA$unique_id, follow$unique_id, nomatch=NA),'death_d']
sub_artA$death_d <- as.Date(sub_artA$death_d, format='%Y-%m-%d')

# Want to make sure that those with missing end dates don't have another regimen after this where the start date of the next one could be used as the end date of the first one
num_artA <- lapply(split(subset(sub_artA, select=c(unique_id)), sub_artA$unique_id),FUN=function(y){
              num_art <- seq(1,nrow(y))
              max_num_art <- max(num_art)
              df <- data.frame('unique_id'=y[1,'unique_id'],
                               'num_art'=num_art,
                               'max_num_art'=rep(max_num_art,times=nrow(y)))
              return(df) })
num_artA.df <- do.call('rbind',num_artA)
sub_artA <- sub_artA[order(sub_artA$unique_id, sub_artA$art_sd),]
num_artA.df <- num_artA.df[order(num_artA.df$unique_id, num_artA.df$num_art),]
sub_artA$num_art <- num_artA.df$num_art
sub_artA$max_num_art <- num_artA.df[match(sub_artA$unique_id, num_artA.df$unique_id, nomatch=NA),'max_num_art']

# Note:  Doesn't seem to be any in the data for which this is true so will just apply l_alive_d to art_ed
sub_artA$art_ed <- ifelse(is.na(sub_artA$art_ed),sub_artA$l_alive_d, sub_artA$art_ed)
sub_artA$art_ed <- as.Date(sub_artA$art_ed, origin='1970-01-01')

sub_artA$days_from_dx_to_death <- ifelse(is.na(sub_artA$tbtx_cont_stop_d) & !is.na(sub_artA$death_d), as.numeric(sub_artA$death_d - sub_artA$tbdiagnosis_d),NA)

sub_artA$tbtx_cont_stop_d <- ifelse(is.na(sub_artA$tbtx_cont_stop_d) & !is.na(sub_artA$death_d) & sub_artA$days_from_dx_to_death <= 180 & 
                                    !is.na(sub_artA$days_from_dx_to_death),sub_artA$death_d,
                                   sub_artA$tbtx_cont_stop_d)
sub_artA$tbtx_cont_stop_d <- as.Date(sub_artA$tbtx_cont_stop_d, origin='1970-01-01')

sub_artA$tbtx_cont_stop_d <- ifelse(is.na(sub_artA$tbtx_cont_stop_d) & as.numeric(sub_artA$l_alive_d - sub_artA$tbdiagnosis_d) <= 365.25,
                                    sub_artA$l_alive_d,sub_artA$tbtx_cont_stop_d)
sub_artA$tbtx_cont_stop_d <- as.Date(sub_artA$tbtx_cont_stop_d, origin='1970-01-01')

sub_artA$tb_relative_to_art_old <- ifelse(!is.na(sub_artA$tbtx_cont_stop_d) & sub_artA$tbtx_cont_stop_d <= sub_artA$art_sd,1,
                                     ifelse(sub_artA$tbdiagnosis_d <= sub_artA$art_sd & !is.na(sub_artA$tbtx_cont_stop_d) & 
                                     sub_artA$tbtx_cont_stop_d > sub_artA$art_sd,0,
                                            ifelse(sub_artA$tbdiagnosis_d > sub_artA$art_sd & sub_artA$tbdiagnosis_d < sub_artA$art_ed & 
                                            !is.na(sub_artA$art_ed),0,
                                                   ifelse(sub_artA$tbdiagnosis_d >= sub_artA$art_ed & !is.na(sub_artA$art_ed),-1,NA))))

sub_artA$tb_relative_to_art <- ifelse(sub_artA$art_sd < sub_artA$tbdiagnosis_d & sub_artA$art_ed <= sub_artA$tbdiagnosis_d,-1,
                                     ifelse(!is.na(sub_artA$tbtx_cont_stop_d) & sub_artA$art_sd > sub_artA$tbtx_cont_stop_d,1,
                                     ifelse(sub_artA$art_sd <= sub_artA$tbdiagnosis_d & !is.na(sub_artA$tbtx_cont_stop_d) & 
                                     sub_artA$art_ed < sub_artA$tbtx_cont_stop_d,0,
                                     ifelse(sub_artA$art_sd <= sub_artA$tbdiagnosis_d & !is.na(sub_artA$tbtx_cont_stop_d) & 
                                     sub_artA$art_ed >= sub_artA$tbtx_cont_stop_d,0,
                                     ifelse(sub_artA$art_sd >= sub_artA$tbdiagnosis_d & !is.na(sub_artA$tbtx_cont_stop_d) & 
                                     sub_artA$art_ed <= sub_artA$tbtx_cont_stop_d,0,
                                     ifelse(sub_artA$art_sd >= sub_artA$tbdiagnosis_d & 
                                     !is.na(sub_artA$tbtx_cont_stop_d) & sub_artA$art_sd < sub_artA$tbtx_cont_stop_d & 
                                     sub_artA$art_ed >= sub_artA$tbtx_cont_stop_d,0,
                                     ifelse(is.na(sub_artA$tbtx_cont_stop_d) & sub_artA$art_sd <= sub_artA$tbdiagnosis & 
                                     sub_artA$art_ed > sub_artA$tbdiagnosis_d,0,
                                     ifelse(sub_artA$art_sd > sub_artA$tbtx_cont_stop_d,1,
                                     ifelse(sub_artA$art_sd >= sub_artA$tbtx_cont_stop_d,1,NA)))))))))


tb_relative_to_artA <- lapply(split(sub_artA[,'tb_relative_to_art'], sub_artA[,'unique_id']), FUN=function(y){
                             y <- y[which(!is.na(y))]
                             if(length(y) > 0){
                                j <- ifelse(any(y[which(!is.na(y))] == 0),0,max(y[which(!is.na(y))]))
                             } else {
                                j <- NA
                             }
                             return(j) })
tb_relative_to_artA.df <- data.frame('unique_id'=names(tb_relative_to_artA),
                                    'tb_relative_to_art'=unlist(tb_relative_to_artA))
ce_tb_tmp4$tb_relative_to_art <- tb_relative_to_artA.df[match(ce_tb_tmp4$unique_id, tb_relative_to_artA.df$unique_id, nomatch=NA),'tb_relative_to_art']
ce_tb_tmp4$tb_relative_to_art <- ifelse(ce_tb_tmp4$unique_id %nin% unique(sub_artA$unique_id),2,ce_tb_tmp4$tb_relative_to_art)
ce_tb_tmp4$tb_relative_to_art.factor <- factor(ce_tb_tmp4$tb_relative_to_art, levels=c(-1,0,1,2), 
                                               labels=c('HAART stopped before TB dx',
                                                        'HAART concurrent with TB trt',
                                                        'HAART started after TB trt stopped',
                                                        'Never on HAART'))

sub_artA$tb_relative_to_art <- tb_relative_to_artA.df[match(sub_artA$unique_id, tb_relative_to_artA.df$unique_id, nomatch=NA),'tb_relative_to_art']
sub_artA$tb_relative_to_art.factor <- factor(sub_artA$tb_relative_to_art, 
                                             levels=c(-1,0,1,2), 
                                             labels=c('HAART stopped before TB dx',
                                                      'HAART concurrent with TB trt',
                                                      'HAART started after TB trt stopped',
                                                      'Never on HAART'))
on_haart_at_tb_dxA <- lapply(split(sub_artA[,c('unique_id','art_sd','art_ed','tbdiagnosis_d','tbtx_cont_stop_d','tb_relative_to_art')],
                                   sub_artA[,'unique_id']),FUN=function(y){
                        id <- y[1,'unique_id']
#                         y <- y[which(!is.na(y[,'tb_relative_to_art']) & y[,'tb_relative_to_art'] == 0),]
                        y <- y[which(!is.na(y[,'tb_relative_to_art'])),]
                        j <- nrow(y)
                        if(length(j) > 0){
                           k <- ifelse(!is.na(y[,'tbtx_cont_stop_d']) & y[,'tb_relative_to_art'] == 0 & y[,'art_sd'] <= y[,'tbdiagnosis_d'] & y[,'art_ed'] > y[,'tbdiagnosis_d'] & y[,'art_ed'] <= y[,'tbtx_cont_stop_d'],1,
                                       ifelse(!is.na(y[,'tbtx_cont_stop_d']) & y[,'tb_relative_to_art'] == 0 & y[,'art_sd'] <= y[,'tbdiagnosis_d'] & y[,'art_ed'] >= y[,'tbtx_cont_stop_d'],1,
                                       ifelse(y[,'tb_relative_to_art'] == 0 & y[,'art_sd'] > y[,'tbdiagnosis_d'] & y[,'art_ed'] < y[,'tbtx_cont_stop_d'],0,
                                       ifelse(y[,'tb_relative_to_art'] == 0 & y[,'art_sd'] > y[,'tbdiagnosis_d'] & y[,'art_ed'] >= y[,'tbtx_cont_stop_d'],0,
                                       ifelse(is.na(y[,'tbtx_cont_stop_d']) & y[,'tb_relative_to_art'] == 0 & y[,'art_sd'] <= y[,'tbdiagnosis_d'] & y[,'art_ed'] > y[,'tbdiagnosis_d'],1,
                                       ifelse(y[,'tb_relative_to_art'] == -1,0,
                                       ifelse(y[,'tb_relative_to_art'] == 1,0,NA)))))))
                           k <- max(k,na.rm=TRUE) # This will always select 1 (on HAART at TB diagnosis if it was ever TRUE)
                           if(k == -Inf) k <- NA
                           df <- data.frame('unique_id'=id,
                                            'k'=k)
                        } else {
                           k <- NA
                           df <- data.frame('unique_id',id,
                                            'k'=k)
                        }
                        return(df)
                        })
on_haart_at_tb_dxA.df <- do.call('rbind',on_haart_at_tb_dxA)
sub_artA$on_haart_at_tb_dx <- on_haart_at_tb_dxA.df[match(sub_artA$unique_id, on_haart_at_tb_dxA.df$unique_id, nomatch=NA),'k']
sub_artA$on_haart_at_tb.factor <- factor(sub_artA$on_haart_at_tb,
                                         levels=c(0,1),
                                         labels=c('Not on HAART at TB diagnosis','On HAART at TB diagnosis'))
ce_tb_tmp4$on_haart_at_tb_dx <- on_haart_at_tb_dxA.df[match(ce_tb_tmp4$unique_id, on_haart_at_tb_dxA.df$unique_id, nomatch=NA),'k']
ce_tb_tmp4$on_haart_at_tb_dx <- ifelse(ce_tb_tmp4$unique_id %nin% on_haart_at_tb_dxA.df$unique_id,0,ce_tb_tmp4$on_haart_at_tb_dx)
ce_tb_tmp4$on_haart_at_tb_dx.factor <- factor(ce_tb_tmp4$on_haart_at_tb,
                                              levels=c(0,1),
                                              labels=c('Not on HAART at TB diagnosis',
                                                       'On HAART at TB diagnosis'))

# Subsetting std_tb to remove those with missing tb_relative_to_art (means their tbtx_cont_stop_d is missing)
ce_tb_tmp5 <- subset(ce_tb_tmp4, !is.na(tb_relative_to_art))  #### Here is where the extra person from brazil gets deleted....

# Also subsetting out all who do not have HAART and TB concurrent
ce_tb_tmp6 <- subset(ce_tb_tmp5,  tb_relative_to_art.factor == 'HAART concurrent with TB trt')



#-------------------------------#
#     Defining baseline CD4     #
#-------------------------------#
ce_tb_tmp6$birth_d <- basic[match(ce_tb_tmp6$unique_id, basic$unique_id, nomatch=NA),'birth_d']
ce_tb_tmp6$birth_d <- as.Date(ce_tb_tmp6$birth_d, format='%Y-%m-%d')
ce_tb_tmp6$age_tb_dx <- as.numeric(ce_tb_tmp6$tbdiagnosis_d - ce_tb_tmp6$birth_d)/365.25

cd4$cd4_d <- as.Date(cd4$cd4_d, format='%Y-%m-%d')
cd4$tbdiagnosis_dA <- ce_tb_tmp6[match(cd4$unique_id, ce_tb_tmp6$unique_id, nomatch=NA),'tbdiagnosis_d']
cd4$date_diffA <- as.numeric(cd4$cd4_d - cd4$tbdiagnosis_dA)
cd4$window_indA <- ifelse(cd4$date_diffA >= -180 & cd4$date_diffA <= 30,1,0)
baseline_cd4A <- lapply(split(cd4[,c('unique_id','window_indA','date_diffA','cd4_v')],cd4[,'unique_id']),FUN=function(y){
                   tmp <- subset(y,window_indA == 1)
                   if(dim(tmp)[1] > 0){
                      min_diff <- min(abs(tmp[,'date_diffA']),na.rm=TRUE)
                      which.row <- which(abs(tmp[,'date_diffA']) == min_diff)
                      b_cd4 <- tmp[which.row,'cd4_v']
                   } else {
                      b_cd4 <- NA
                   }
                   df <- data.frame('unique_id'=y[1,'unique_id'],
                                    'b_cd4'=b_cd4)
                      return(df) })
baseline_cd4_dfA <- do.call('rbind',baseline_cd4A)
baseline_cd4_dfA <- subset(baseline_cd4_dfA, unique_id %in% ce_tb_tmp6$unique_id)

# Adding in baseline CD4 to std_tb
ce_tb_tmp6$baseline_cd4 <- baseline_cd4_dfA[match(ce_tb_tmp6$unique_id, baseline_cd4_dfA$unique_id, nomatch=NA),'b_cd4']





# Time to death or censoring
ce_tb_tmp6$l_alive_d <- follow[match(ce_tb_tmp6$unique_id, follow$unique_id, nomatch=NA),'l_alive_d']
ce_tb_tmp6$l_alive_d <- as.Date(ce_tb_tmp6$l_alive_d, format='%Y-%m-%d')
ce_tb_tmp6$time_to_death <- as.numeric(ce_tb_tmp6$l_alive_d - ce_tb_tmp6$tbdiagnosis_d)
ce_tb_tmp6$time_to_death_yrs <- ce_tb_tmp6$time_to_death/365.25



ce_tb_tmp6_original <- ce_tb_tmp6

ce_tb_tmp7 <- subset(ce_tb_tmp6, time_to_death > 0)

# Removing subjects whose age_tb_dx < 18 (Added 7/6/2021):
ce_tb_tmp8 <- ce_tb_tmp7[!is.na(ce_tb_tmp7$age_tb_dx) & ce_tb_tmp7$age_tb_dx >= 18.0,]

ce_tb_tmp8$all_patients <- 1
ce_tb_tmp8$tbtx_intermittency_cat <- ifelse(!is.na(ce_tb_tmp8$tbtx_intermittency) &
                                            ce_tb_tmp8$tbtx_intermittency == 0,0,
                                        ifelse(!is.na(ce_tb_tmp8$tbtx_intermittency),1,NA))
ce_tb_tmp8$tbtx_intermittency_cat.factor <- factor(ce_tb_tmp8$tbtx_intermittency_cat,
                                                   levels=c(0,1),
                                                   labels=c('2x/wk','>2x/wk'))
ce_tb_tmp8$intermittency_data <- ifelse(is.na(ce_tb_tmp8$tbtx_intermittency),0,1)
ce_tb_tmp8$intermittency_data.factor <- factor(ce_tb_tmp8$intermittency_data,
                                               levels=c(0,1),
                                               labels=c('No intermittency data','Intermittency data'))
## There are 19 subjects whose tbtx_intermittency = 5x/wk (19, all from Honduras).  Will code them in with the 7x/wk and make note of it
ce_tb_tmp8$tbtx_intermittency_original <- ce_tb_tmp8$tbtx_intermittency
ce_tb_tmp8$tbtx_intermittency_original.factor <- factor(ce_tb_tmp8$tbtx_intermittency_original, 
                                                        levels=c(0,1,2,3),
                                                        labels=c('2x/wk',
                                                                 '3x/wk',
                                                                 '5x/wk',
                                                                 '7x/wk'))
ce_tb_tmp8$tbtx_intermittency <- ifelse(ce_tb_tmp8$tbtx_intermittency %in% c(2,3),3,ce_tb_tmp8$tbtx_intermittency)
ce_tb_tmp8$tbtx_intermittency.factor <- factor(ce_tb_tmp8$tbtx_intermittency, 
                                               levels=c(0,1,3), 
                                               labels=c('2x/wk',
                                                        '3x/wk',
                                                        '5-7x/wk'))
ce_tb_tmp8$tbtx_intermittency_2_7 <- ifelse(ce_tb_tmp8$tbtx_intermittency_original == 0,0,ifelse(ce_tb_tmp8$tbtx_intermittency_original == 3,1,NA))
ce_tb_tmp8$tbtx_intermittency_2_7.factor <- factor(ce_tb_tmp8$tbtx_intermittency_2_7, 
                                                   levels=c(0,1), 
                                                   labels=c('2x/wk','7x/wk'))



